@@ -57,6 +57,7 @@ use error::*;
 use plutus::*;
 use metadata::*;
 use utils::*;
+use std::cmp::Ordering;
 
 type DeltaCoin = Int;
 
@@ -97,6 +98,7 @@ type Slot = u32;
 pub struct Transaction {
     body: TransactionBody,
     witness_set: TransactionWitnessSet,
+    is_valid: bool,
     auxiliary_data: Option<AuxiliaryData>,
 }
 
@@ -112,10 +114,18 @@ impl Transaction {
         self.witness_set.clone()
     }
 
+    pub fn is_valid(&self) -> bool {
+        self.is_valid.clone()
+    }
+
     pub fn auxiliary_data(&self) -> Option<AuxiliaryData> {
         self.auxiliary_data.clone()
     }
 
+    pub fn set_is_valid(&mut self, valid: bool) {
+        self.is_valid = valid
+    }
+
     pub fn new(
         body: &TransactionBody,
         witness_set: &TransactionWitnessSet,
@@ -124,6 +134,7 @@ impl Transaction {
         Self {
             body: body.clone(),
             witness_set: witness_set.clone(),
+            is_valid: true,
             auxiliary_data: auxiliary_data.clone(),
         }
     }
@@ -2403,9 +2414,26 @@ impl HeaderBody {
 
 
 #[wasm_bindgen]
-#[derive(Clone, Debug, Eq, Ord, PartialEq, PartialOrd)]
+#[derive(Clone, Debug, Eq, PartialEq)]
 pub struct AssetName(Vec<u8>);
 
+impl Ord for AssetName {
+    fn cmp(&self, other: &Self) -> Ordering {
+        // Implementing canonical CBOR order for asset names,
+        // as they might be of different length.
+        return match self.0.len().cmp(&other.0.len()) {
+            Ordering::Equal => self.0.cmp(&other.0),
+            x => x,
+        };
+    }
+}
+
+impl PartialOrd for AssetName {
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+        Some(self.cmp(other))
+    }
+}
+
 to_from_bytes!(AssetName);
 
 #[wasm_bindgen]
@@ -2698,4 +2726,42 @@ mod tests {
 
         assert_eq!(hex::encode(&script_hash.to_bytes()), "187b8d3ddcb24013097c003da0b8d8f7ddcf937119d8f59dccd05a0f");
     }
+
+    #[test]
+    fn asset_name_ord() {
+
+        let name1 = AssetName::new(vec![0u8, 1, 2, 3]).unwrap();
+        let name11 = AssetName::new(vec![0u8, 1, 2, 3]).unwrap();
+
+        let name2 = AssetName::new(vec![0u8, 4, 5, 6]).unwrap();
+        let name22 = AssetName::new(vec![0u8, 4, 5, 6]).unwrap();
+
+        let name3 = AssetName::new(vec![0u8, 7, 8]).unwrap();
+        let name33 = AssetName::new(vec![0u8, 7, 8]).unwrap();
+
+        assert_eq!(name1.cmp(&name2), Ordering::Less);
+        assert_eq!(name2.cmp(&name1), Ordering::Greater);
+        assert_eq!(name1.cmp(&name3), Ordering::Greater);
+        assert_eq!(name2.cmp(&name3), Ordering::Greater);
+        assert_eq!(name3.cmp(&name1), Ordering::Less);
+        assert_eq!(name3.cmp(&name2), Ordering::Less);
+
+        assert_eq!(name1.cmp(&name11), Ordering::Equal);
+        assert_eq!(name2.cmp(&name22), Ordering::Equal);
+        assert_eq!(name3.cmp(&name33), Ordering::Equal);
+
+        let mut map = Assets::new();
+        map.insert(&name2, &to_bignum(1));
+        map.insert(&name1, &to_bignum(1));
+        map.insert(&name3, &to_bignum(1));
+
+        assert_eq!(map.keys(), AssetNames(vec![name3, name1, name2]));
+
+        let mut map2 = MintAssets::new();
+        map2.insert(&name11, Int::new_i32(1));
+        map2.insert(&name33, Int::new_i32(1));
+        map2.insert(&name22, Int::new_i32(1));
+
+        assert_eq!(map2.keys(), AssetNames(vec![name33, name11, name22]));
+    }
 }
